translations = {
    "ru": {
        "read_more": "Далее",
        "no_posts_found": "Ничего не найдено.",
        "pagination_first_page": "к новым",
        "pagination_last_page": "к старым",
        "tags_filter_header": "Нашли постов:",
        "article_similar_header": "Похожее",
        "last_posts_header": "Последние посты",
        "similarity_score_tooltip": "Уровень похожести",
    },
    "en": {
        "read_more": "Read more",
        "no_posts_found": "Nothing found.",
        "pagination_first_page": "to new",
        "pagination_last_page": "to old",
        "tags_filter_header": "Found posts:",
        "article_similar_header": "Similar",
        "last_posts_header": "Last posts",
        "similarity_score_tooltip": "Similarity score",
    },
}
